//*****************************************************************************
// Copyright (c) 2016-2020, Intel Corporation
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// - Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// - Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.
//*****************************************************************************

/*
 * This header file contains single argument bitwise functions definitions
 *
 * Macro `MACRO_2ARG_1TYPE_OP` must be defined before usage
 *
 * Parameters:
 * - public name of the function and kernel name
 * - operation used to calculate the result
 *
 */

#ifndef MACRO_2ARG_1TYPE_OP
#error "MACRO_2ARG_1TYPE_OP is not defined"
#endif

#ifdef _SECTION_DOCUMENTATION_GENERATION_

#define MACRO_2ARG_1TYPE_OP(__name__, __operation__)                                                                    \
    /** @ingroup BACKEND_API                                                                                         */ \
    /** @brief Element wise operation function __name__                                                              */ \
    /**                                                                                                              */ \
    /** Function "__name__" executes operator "__operation__" over corresponding elements of input arrays            */ \
    /**                                                                                                              */ \
    /** @param[out] result1  Output array.                                                                           */ \
    /** @param[in]  array1   Input array 1.                                                                          */ \
    /** @param[in]  size1    Number of elements in @ref array1                                                       */ \
    /** @param[in]  array2   Input array 2.                                                                          */ \
    /** @param[in]  size2    Number of elements in @ref array2                                                       */ \
    template <typename _DataType>                                                                                       \
    void __name__(void* result1, const void* array1, const size_t size1, const void* array2, const size_t size2);

#endif

MACRO_2ARG_1TYPE_OP(dpnp_bitwise_and_c, input_elem1& input_elem2)
MACRO_2ARG_1TYPE_OP(dpnp_bitwise_or_c, input_elem1 | input_elem2)
MACRO_2ARG_1TYPE_OP(dpnp_bitwise_xor_c, input_elem1 ^ input_elem2)
MACRO_2ARG_1TYPE_OP(dpnp_left_shift_c, input_elem1 << input_elem2)
MACRO_2ARG_1TYPE_OP(dpnp_right_shift_c, input_elem1 >> input_elem2)

#undef MACRO_2ARG_1TYPE_OP
